@@ -19,7 +19,10 @@ _mut_id = itertools.count(1).__next__
 
 def map_hash(o):
     x = hash(o)
-    return (x & 0xffffffff) ^ ((x >> 32) & 0xffffffff)
+    if sys.hash_info.width > 32:
+        return (x & 0xffffffff) ^ ((x >> 32) & 0xffffffff)
+    else:
+        return x
 
 
 def map_mask(hash, shift):
@@ -47,6 +50,15 @@ W_EMPTY, W_NEWNODE, W_NOT_FOUND = range(3)
 void = object()
 
 
+class _Unhashable:
+    __slots__ = ()
+    __hash__ = None
+
+
+_NULL = _Unhashable()
+del _Unhashable
+
+
 class BitmapNode:
 
     def __init__(self, size, bitmap, array, mutid):
@@ -70,7 +82,7 @@ class BitmapNode:
             key_or_null = self.array[key_idx]
             val_or_node = self.array[val_idx]
 
-            if key_or_null is None:
+            if key_or_null is _NULL:
                 sub_node, added = val_or_node.assoc(
                     shift + 5, hash, key, val, mutid)
                 if val_or_node is sub_node:
@@ -111,12 +123,12 @@ class BitmapNode:
                     mutid)
 
             if mutid and mutid == self.mutid:
-                self.array[key_idx] = None
+                self.array[key_idx] = _NULL
                 self.array[val_idx] = sub_node
                 return self, True
             else:
                 ret = self.clone(mutid)
-                ret.array[key_idx] = None
+                ret.array[key_idx] = _NULL
                 ret.array[val_idx] = sub_node
                 return ret, True
 
@@ -153,7 +165,7 @@ class BitmapNode:
         key_or_null = self.array[key_idx]
         val_or_node = self.array[val_idx]
 
-        if key_or_null is None:
+        if key_or_null is _NULL:
             return val_or_node.find(shift + 5, hash, key)
 
         if key == key_or_null:
@@ -173,7 +185,7 @@ class BitmapNode:
         key_or_null = self.array[key_idx]
         val_or_node = self.array[val_idx]
 
-        if key_or_null is None:
+        if key_or_null is _NULL:
             res, sub_node = val_or_node.without(shift + 5, hash, key, mutid)
 
             if res is W_EMPTY:
@@ -182,7 +194,7 @@ class BitmapNode:
             elif res is W_NEWNODE:
                 if (type(sub_node) is BitmapNode and
                         sub_node.size == 2 and
-                        sub_node.array[0] is not None):
+                        sub_node.array[0] is not _NULL):
 
                     if mutid and mutid == self.mutid:
                         self.array[key_idx] = sub_node.array[0]
@@ -231,7 +243,7 @@ class BitmapNode:
         for i in range(0, self.size, 2):
             key_or_null = self.array[i]
 
-            if key_or_null is None:
+            if key_or_null is _NULL:
                 val_or_node = self.array[i + 1]
                 yield from val_or_node.keys()
             else:
@@ -242,7 +254,7 @@ class BitmapNode:
             key_or_null = self.array[i]
             val_or_node = self.array[i + 1]
 
-            if key_or_null is None:
+            if key_or_null is _NULL:
                 yield from val_or_node.values()
             else:
                 yield val_or_node
@@ -252,7 +264,7 @@ class BitmapNode:
             key_or_null = self.array[i]
             val_or_node = self.array[i + 1]
 
-            if key_or_null is None:
+            if key_or_null is _NULL:
                 yield from val_or_node.items()
             else:
                 yield key_or_null, val_or_node
@@ -269,8 +281,8 @@ class BitmapNode:
 
             pad = '    ' * (level + 2)
 
-            if key_or_null is None:
-                buf.append(pad + 'None:')
+            if key_or_null is _NULL:
+                buf.append(pad + 'NULL:')
                 val_or_node.dump(buf, level + 2)
             else:
                 buf.append(pad + '{!r}: {!r}'.format(key_or_null, val_or_node))
@@ -328,7 +340,7 @@ class CollisionNode:
 
         else:
             new_node = BitmapNode(
-                2, map_bitpos(self.hash, shift), [None, self], mutid)
+                2, map_bitpos(self.hash, shift), [_NULL, self], mutid)
             return new_node.assoc(shift, hash, key, val, mutid)
 
     def without(self, shift, hash, key, mutid):
@@ -433,7 +445,17 @@ class MapItems:
 
 class Map:
 
-    def __init__(self, col=None, **kw):
+    def __init__(self, *args, **kw):
+        if not args:
+            col = None
+        elif len(args) == 1:
+            col = args[0]
+        else:
+            raise TypeError(
+                "immutables.Map expected at most 1 arguments, "
+                "got {}".format(len(args))
+            )
+
         self.__count = 0
         self.__root = BitmapNode(0, 0, [], 0)
         self.__hash = -1
@@ -483,8 +505,18 @@ class Map:
 
         return True
 
-    def update(self, col=None, **kw):
+    def update(self, *args, **kw):
+        if not args:
+            col = None
+        elif len(args) == 1:
+            col = args[0]
+        else:
+            raise TypeError(
+                "update expected at most 1 arguments, got {}".format(len(args))
+            )
+
         it = None
+
         if col is not None:
             if hasattr(col, 'items'):
                 it = iter(col.items())
@@ -622,8 +654,7 @@ class Map:
         items = []
         for key, val in self.items():
             items.append("{!r}: {!r}".format(key, val))
-        return '<immutables.Map({{{}}}) at 0x{:0x}>'.format(
-            ', '.join(items), id(self))
+        return 'immutables.Map({{{}}})'.format(', '.join(items))
 
     def __dump__(self):  # pragma: no cover
         buf = []
@@ -721,7 +752,16 @@ class MapMutation:
         else:
             return True
 
-    def update(self, col=None, **kw):
+    def update(self, *args, **kw):
+        if not args:
+            col = None
+        elif len(args) == 1:
+            col = args[0]
+        else:
+            raise TypeError(
+                "update expected at most 1 arguments, got {}".format(len(args))
+            )
+
         if self.__mutid == 0:
             raise ValueError('mutation {!r} has been finished'.format(self))
 
@@ -740,8 +780,7 @@ class MapMutation:
                 it = iter(kw.items())
 
         if it is None:
-
-            return self
+            return
 
         root = self.__root
         count = self.__count
@@ -783,8 +822,7 @@ class MapMutation:
         items = []
         for key, val in self.__root.items():
             items.append("{!r}: {!r}".format(key, val))
-        return '<immutables.MapMutation({{{}}}) at 0x{:0x}>'.format(
-            ', '.join(items), id(self))
+        return 'immutables.MapMutation({{{}}})'.format(', '.join(items))
 
     def __len__(self):
         return self.__count
