@@ -7,88 +7,12 @@ import unittest
 import weakref
 
 from immutables.map import Map as PyMap
-
-
-class HashKey:
-    _crasher = None
-
-    def __init__(self, hash, name, *, error_on_eq_to=None):
-        assert hash != -1
-        self.name = name
-        self.hash = hash
-        self.error_on_eq_to = error_on_eq_to
-
-    def __repr__(self):
-        if self._crasher is not None and self._crasher.error_on_repr:
-            raise ReprError
-        return '<Key name:{} hash:{}>'.format(self.name, self.hash)
-
-    def __hash__(self):
-        if self._crasher is not None and self._crasher.error_on_hash:
-            raise HashingError
-
-        return self.hash
-
-    def __eq__(self, other):
-        if not isinstance(other, HashKey):
-            return NotImplemented
-
-        if self._crasher is not None and self._crasher.error_on_eq:
-            raise EqError
-
-        if self.error_on_eq_to is not None and self.error_on_eq_to is other:
-            raise ValueError('cannot compare {!r} to {!r}'.format(self, other))
-        if other.error_on_eq_to is not None and other.error_on_eq_to is self:
-            raise ValueError('cannot compare {!r} to {!r}'.format(other, self))
-
-        return (self.name, self.hash) == (other.name, other.hash)
-
-
-class KeyStr(str):
-
-    def __hash__(self):
-        if HashKey._crasher is not None and HashKey._crasher.error_on_hash:
-            raise HashingError
-        return super().__hash__()
-
-    def __eq__(self, other):
-        if HashKey._crasher is not None and HashKey._crasher.error_on_eq:
-            raise EqError
-        return super().__eq__(other)
-
-    def __repr__(self, other):
-        if HashKey._crasher is not None and HashKey._crasher.error_on_repr:
-            raise ReprError
-        return super().__eq__(other)
-
-
-class HashKeyCrasher:
-
-    def __init__(self, *, error_on_hash=False, error_on_eq=False,
-                 error_on_repr=False):
-        self.error_on_hash = error_on_hash
-        self.error_on_eq = error_on_eq
-        self.error_on_repr = error_on_repr
-
-    def __enter__(self):
-        if HashKey._crasher is not None:
-            raise RuntimeError('cannot nest crashers')
-        HashKey._crasher = self
-
-    def __exit__(self, *exc):
-        HashKey._crasher = None
-
-
-class HashingError(Exception):
-    pass
-
-
-class EqError(Exception):
-    pass
-
-
-class ReprError(Exception):
-    pass
+from immutables._testutils import EqError
+from immutables._testutils import HashKey
+from immutables._testutils import HashKeyCrasher
+from immutables._testutils import HashingError
+from immutables._testutils import KeyStr
+from immutables._testutils import ReprError
 
 
 class BaseMapTest:
@@ -240,8 +164,6 @@ class BaseMapTest:
         #                             <Key name:E hash:362244>: 'e'
         #     <Key name:B hash:101>: 'b'
 
-
-
     def test_map_stress_01(self):
         COLLECTION_SIZE = 7000
         TEST_ITERS_EVERY = 647
@@ -332,6 +254,40 @@ class BaseMapTest:
             self.assertEqual(len(h), 0)
             self.assertEqual(list(h.items()), [])
 
+    def test_map_collision_3(self):
+        # Test that iteration works with the deepest tree possible.
+
+        C = HashKey(0b10000000_00000000_00000000_00000000, 'C')
+        D = HashKey(0b10000000_00000000_00000000_00000000, 'D')
+
+        E = HashKey(0b00000000_00000000_00000000_00000000, 'E')
+
+        h = self.Map()
+        h = h.set(C, 'C')
+        h = h.set(D, 'D')
+        h = h.set(E, 'E')
+
+        # BitmapNode(size=2 count=1 bitmap=0b1):
+        #   NULL:
+        #     BitmapNode(size=2 count=1 bitmap=0b1):
+        #       NULL:
+        #         BitmapNode(size=2 count=1 bitmap=0b1):
+        #           NULL:
+        #             BitmapNode(size=2 count=1 bitmap=0b1):
+        #               NULL:
+        #                 BitmapNode(size=2 count=1 bitmap=0b1):
+        #                   NULL:
+        #                     BitmapNode(size=2 count=1 bitmap=0b1):
+        #                       NULL:
+        #                         BitmapNode(size=4 count=2 bitmap=0b101):
+        #                           <Key name:E hash:0>: 'E'
+        #                           NULL:
+        #                             CollisionNode(size=4 id=0x107a24520):
+        #                               <Key name:C hash:2147483648>: 'C'
+        #                               <Key name:D hash:2147483648>: 'D'
+
+        self.assertEqual({k.name for k in h.keys()}, {'C', 'D', 'E'})
+
     def test_map_stress_02(self):
         COLLECTION_SIZE = 20000
         TEST_ITERS_EVERY = 647
@@ -926,11 +882,10 @@ class BaseMapTest:
 
     def test_repr_1(self):
         h = self.Map()
-        self.assertTrue(repr(h).startswith('<immutables.Map({}) at 0x'))
+        self.assertEqual(repr(h), 'immutables.Map({})')
 
         h = h.set(1, 2).set(2, 3).set(3, 4)
-        self.assertTrue(repr(h).startswith(
-            '<immutables.Map({1: 2, 2: 3, 3: 4}) at 0x'))
+        self.assertEqual(repr(h), 'immutables.Map({1: 2, 2: 3, 3: 4})')
 
     def test_repr_2(self):
         h = self.Map()
@@ -960,8 +915,7 @@ class BaseMapTest:
         h = h.set(k, 1)
         k.val = h
 
-        self.assertTrue(repr(h).startswith(
-            '<immutables.Map({{...}: 1}) at 0x'))
+        self.assertEqual(repr(h), 'immutables.Map({{...}: 1})')
 
     def test_hash_1(self):
         h = self.Map()
@@ -1045,8 +999,7 @@ class BaseMapTest:
         h = h.set('a', 1)
         hm1 = h.mutate()
 
-        self.assertTrue(repr(hm1).startswith(
-            "<immutables.MapMutation({'a': 1})"))
+        self.assertEqual(repr(hm1), "immutables.MapMutation({'a': 1})")
 
         with self.assertRaisesRegex(TypeError, 'unhashable type'):
             hash(hm1)
@@ -1319,7 +1272,6 @@ class BaseMapTest:
             # node to be converted into an array node
             h = h.set(HashKey(i, i), i)
 
-
         h = h.set(HashKey(18, '18-collision'), 18)
 
         with h.mutate() as m:
@@ -1432,7 +1384,9 @@ class BaseMapTest:
         with self.assertRaisesRegex(TypeError, "can('t|not) pickle"):
             pickle.dumps(h.mutate())
 
-    @unittest.skipIf(sys.version_info < (3, 7, 0), "__class_getitem__ is not available")
+    @unittest.skipIf(
+        sys.version_info < (3, 7, 0), "__class_getitem__ is not available"
+    )
     def test_map_is_subscriptable(self):
         self.assertIs(self.Map[int, str], self.Map)
 
